package net.wanji.web.task;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.feign.pojo.result.JsonViewObject;
import net.wanji.feign.service.UtcFeignClients;
import net.wanji.web.common.util.StringUtils;
import net.wanji.web.entity.TBaseCrossInfo;
import net.wanji.web.entity.TDeviceStatusInfo;
import net.wanji.web.mapper.TDeviceStatusMapper;
import net.wanji.web.service.SituationDetectionService;
import net.wanji.web.vo.situationDetection.CommonEventAlarmOutVo;
import net.wanji.web.websocket.CommonEventWarnWebSocketServer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author duanruiming
 * @date 2023/01/06 10:45
 */
@Component
@Slf4j
public class CommonEventWarnTask {

    @Autowired
    private UtcFeignClients utcFeignClients;

    @Autowired
    private SituationDetectionService situationDetectionService;

    @Autowired
    private TDeviceStatusMapper tDeviceStatusMapper;

    @Scheduled(fixedRate = 1 * 60 * 1000)
    public void pushEvenWarnData() {
        try {
            ConcurrentHashMap<String, CommonEventWarnWebSocketServer> evenWarnWebSocketMap = CommonEventWarnWebSocketServer.getEvenWarnWebSocketMap();
            if (!evenWarnWebSocketMap.isEmpty()) {
                for (Map.Entry<String, CommonEventWarnWebSocketServer> entry : evenWarnWebSocketMap.entrySet()) {
                    String key = entry.getKey();
                    getCommonEnventWarnData(key);
                }
            }
        } catch (Exception e) {
            log.error("统一推送事件告警数据失败！", e);
        }
    }

    private void getCommonEnventWarnData(String key) throws IOException {
        // 信号机故障
        JsonViewObject signalWarnData = getSignalWarnData();
        if (Objects.nonNull(signalWarnData)) {
            CommonEventWarnWebSocketServer.sendInfo(JSONObject.toJSONString(signalWarnData), key);
        }
        // todo 数据中心获取路口告警
        JsonViewObject crossWarnData = getCrossWarnData();
        // todo 数据中心获取事件告警
        JsonViewObject crossEventWarnData = getCrossEventWarnData();
    }

    private JsonViewObject getCrossEventWarnData() {
        return null;
    }

    private JsonViewObject getCrossWarnData() {
        return null;
    }

    /**
     * 信号机故障告警数据
     *
     * @return
     */
    private JsonViewObject getSignalWarnData() {
        List<CommonEventAlarmOutVo> results = new ArrayList<>();

        if (true) {
            CommonEventAlarmOutVo commonEventAlarmOutVo = new CommonEventAlarmOutVo();
            commonEventAlarmOutVo.setEventType(3);
            commonEventAlarmOutVo.setEventId("14Q3709IOR0");
            commonEventAlarmOutVo.setEventName("相融路与蠡太路交叉口");
            commonEventAlarmOutVo.setStatus("3");
            commonEventAlarmOutVo.setCoordinate("120.630807,31.422317");

            CommonEventAlarmOutVo commonEventAlarmOutVo1 = new CommonEventAlarmOutVo();
            commonEventAlarmOutVo1.setEventType(3);
            commonEventAlarmOutVo1.setEventId("8");
            commonEventAlarmOutVo1.setEventName("青龙港路与云集街交叉口");
            commonEventAlarmOutVo1.setStatus("3");
            commonEventAlarmOutVo1.setCoordinate("120.640305,31.418544");
            results.add(commonEventAlarmOutVo);
            LocalDateTime now = LocalDateTime.now();
            JsonViewObject jsonViewObject = JsonViewObject.newInstance();
            jsonViewObject.setTimestamp(now.toInstant(ZoneOffset.of("+8")).toEpochMilli());
            jsonViewObject.setContent(results);
            return jsonViewObject;
        }

        JsonViewObject jsonViewObject = utcFeignClients.runningStatusAlarm();
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机告警UTC服务调用异常！");
        }
        LocalDateTime now = LocalDateTime.now();
        String formatNow = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss").format(now);
        List<Map<String, Object>> content = (List<Map<String, Object>>) jsonViewObject.getContent();
        content.forEach(map -> {
            Map<String, Object> result = map;
            String signalId = (String) result.get("signalId");
            String crossId = (String) result.get("crossId");
            Integer currentFaultType = (Integer) result.get("faultType");
            // todo 配合测试提供数据， 后续删除
            if (Objects.equals(crossId, "13FNK0C6790")) {
                currentFaultType = 1;
            }

            String coordinate = "";
            int faultTypeDB = 0;
            if (StringUtils.isNotEmpty(crossId)) {
                TBaseCrossInfo tBaseCrossInfo = situationDetectionService.selectCrossInfoById(crossId);
                if (Objects.nonNull(tBaseCrossInfo)) {
                    coordinate = tBaseCrossInfo.getLocation();
                }
                QueryWrapper<TDeviceStatusInfo> queryWrapper = new QueryWrapper<>();
                TDeviceStatusInfo tDeviceStatusInfo = tDeviceStatusMapper.selectOne(queryWrapper.eq("code", crossId));
                if (Objects.nonNull(tDeviceStatusInfo)) {
                    faultTypeDB = tDeviceStatusInfo.getFaultType();
                }
            }
            if (Objects.nonNull(currentFaultType) && currentFaultType != faultTypeDB) {
                CommonEventAlarmOutVo commonEventAlarmOutVo = new CommonEventAlarmOutVo();
                commonEventAlarmOutVo.setEventType(3);
                commonEventAlarmOutVo.setEventId(signalId);
                commonEventAlarmOutVo.setEventName("信号机故障");// todo 信号机厂商
                commonEventAlarmOutVo.setEventDesc("信号机故障");
                commonEventAlarmOutVo.setStatus(String.valueOf(currentFaultType));
                commonEventAlarmOutVo.setStartTime(formatNow);
                commonEventAlarmOutVo.setCoordinate(coordinate);
                results.add(commonEventAlarmOutVo);
            }
        });
        // 无告警数据
        if (CollectionUtils.isEmpty(results)) {
            return null;
        }
        jsonViewObject.setTimestamp(now.toInstant(ZoneOffset.of("+8")).toEpochMilli());
        jsonViewObject.setContent(results);
        return jsonViewObject;
    }
}
